package simulator;

import java.util.Hashtable;
import java.util.Random;
import java.util.concurrent.DelayQueue;


public class Network extends Thread{
	
	private DelayQueue<Message> queue; //queue of messages to send
	protected Hashtable<Integer,Peer> peers; //set of peers
	protected int nextAddress; //address assigned to the next peer 
	protected Random generator; //random number generator
	protected int[] messageCounters; //statistics 
	protected int delay;
	public static final int EXPECTEDPEERS = 100; //expected number of peers
	
	
	
	/*build a network without hosts*/
	public Network(){
		peers = new Hashtable<Integer,Peer>(EXPECTEDPEERS);
		nextAddress = 1;
		generator = new Random();
		messageCounters = new int[Message.MESSAGETYPES];
		queue = new DelayQueue<Message>();
		delay = 0;
		this.start();
	}
	
	/* Set expected dealy for message for messages */
	public void setAverageDelay(int delay) {
		this.delay = delay;
	}
	
	
	
	/*Get a random address of a peer registred in the network*/
	public int getRandomPeerAddress() {
	    boolean found = false;
	    int randomAddress = 0;
	    while (!found) {
	    	randomAddress = generator.nextInt(nextAddress-1) + 1;
	    	if (peers.containsKey(randomAddress)){
	    		found = true;
	    	}
	    }
	    return randomAddress;
	}
	
	
	/*Main task of a network: it picks a message and delivers it*/
	public void run() {
		Message msg=null;
		System.out.println("Partito.");
		while (true) {
		    try {
		        msg = (Message)queue.take();
		    } catch (InterruptedException e) {e.printStackTrace();};

		    Peer peerDest = (Peer)peers.get(msg.getDestination());
	        if (peerDest != null) {   //host found
	            peerDest.receiveMessage(msg);
	        }
		}
	}
	
	
	/* Send a message to a peer. If the peer does not exist the message is lost */
	/* The message type can be specified for statistics */
	public void sendMessage(Message m) {
	    messageCounters[m.type]++;
	    if (delay > 0)
	        m.setDelay(this.generator.nextInt(delay * 2));
	    else
	    	m.setDelay(0);

        queue.offer(m);
        /*Debug*/
//        System.out.println("Sending Message from " + m.field2 + " to " + m.getDestination());
//        System.out.println("Message TYPE = " + m.type);
//        System.out.println("> TTL = " + m.field3);
	}
	
	
	/* Add a host to the network. Returns the address */
	public int addHost(Peer peer) {
		int address = nextAddress;
		nextAddress = nextAddress + 1;
		
		peers.put(address, peer);
		
		return address;
	}
	
	
	/* Read statistics */
	public int readCounter(int type) {
	    return messageCounters[type];
	}
	
	/*Reset statistics */
	public void resetCounters() {
	   for (int i = 0; i < Message.MESSAGETYPES; i++) {
		   messageCounters[i] = 0;
	   }
	}

}
